/*
 * This file is part of Programming with C# Sockets. 
 * Copyright (C) 2005 - 2010 CodePlanet. All rights reserved. 
 * For more informations, visit http://www.codeplanet.eu/.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Collections;

namespace CodePlanet.Articles.ProgrammingSockets
{
    public partial class ClientForm : Form
    {
        public ClientForm()
        {
            InitializeComponent();
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            // Sind wir bereits verbunden?
            if (_clientSocket != null && _clientSocket.Connected) {
                _clientSocket.Close();
            }

            try {

                // Verwende den angegebenen Host, ansonsten nimm den lokalen Host
                string server = (textBoxIPAddress.Text.Length > 0) ? textBoxIPAddress.Text : "::1";
                // Verwende den angegebenen Port, ansonsten nimm den Standardwert 7
                int servPort = (textBoxPort.Text.Length > 0) ? Convert.ToInt32(textBoxPort.Text) : 3663;

                IPAddress servIPAddress = IPAddress.Parse(server);
                IPEndPoint servEndPoint = new IPEndPoint(servIPAddress, servPort);               

                // Erzeuge neuen IPv4 Tcp Socket
                _clientSocket = new Socket(servIPAddress.AddressFamily, SocketType.Stream, ProtocolType.Tcp);

                // Verbinde...
                _clientSocket.Connect(servEndPoint);

                // Zeige dem User das wir verbunden sind
                if (_clientSocket.Connected) {
                    MessageBox.Show("Connected to " + server, "MessageBox",
                                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            } catch (SocketException se) {
                MessageBox.Show(se.Message);
            }
        }

        private void buttonDisconnect_Click(object sender, EventArgs e)
        {
            if (_clientSocket == null) {
                MessageBox.Show("You must first connect to a Server!", "MessageBox",
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            // Schliee den Socket
            if (_clientSocket.Connected) {
                _clientSocket.Shutdown(SocketShutdown.Both);
            }

            _clientSocket.Close();

            MessageBox.Show("Disconnected!", "MessageBox",
                            MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void buttonSend_Click(object sender, EventArgs e)
        {
            if (_clientSocket == null || !_clientSocket.Connected) {
                MessageBox.Show("You must first connect to a Server!", "MessageBox",
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            try {
                // Sende Daten
                byte[] byteBuffer = System.Text.Encoding.ASCII.GetBytes(textBoxSend.Text);
                _clientSocket.Send(byteBuffer, 0, byteBuffer.Length, SocketFlags.None);
            } catch (SocketException se) {
                MessageBox.Show(se.Message);
            }
        }

        private void buttonReceive_Click(object sender, EventArgs e)
        {
            if (_clientSocket == null || !_clientSocket.Connected) {
                MessageBox.Show("You must first connect to a Server!", "MessageBox",
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            try {
                // Empfange Daten
                byte[] byteBuffer = new byte[2048];
                int bytesRcvd = _clientSocket.Receive(byteBuffer);
                char[] chars = new char[bytesRcvd];

                Decoder dec = Encoding.UTF8.GetDecoder();
                int charLen = dec.GetChars(byteBuffer, 0, bytesRcvd, chars, 0);
                string newData = new string(chars);
                textBoxReceive.Text = newData;
            } catch (SocketException se) {
                MessageBox.Show(se.Message);
            }
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("IExplore", "http://www.codeplanet.eu");
        }

        private Socket _clientSocket;
    }
}